@@ -7,6 +7,7 @@ import time
 import struct
 from collections import deque
 import inspect
+import copy
 
 from .globaling import *
 
@@ -59,20 +60,36 @@ class Act(object):
         """
         return (self.actor(**self.parms))
 
-
     def expose(self):
         """Show Actor and parms"""
         console.terse("Act Actor {0} Parms {1}\n".format(self.actor, self.parms))
         if self.actor:
             self.actor.expose()
+            
+    def clone(self, clones):
+        """ Return clone of self in support of framer cloning
+            clones is dict whose items keys are original framer names
+            and values are duples of (original,clone) framer references
+        """
+        clone = Act(actor=self.actor.clone(), parms=copy.copy(self.parms))
+        clone.cloneParms(clones)
+        return clone
+    
+    def cloneParms(self, clones):
+        """ Fixup parms in actor for framer cloning.
+            clones is dict whose items keys are original framer names
+            and values are are duples of (original,clone) framer references
+            
+        """
+        parms = self.actor.cloneParms(parms=self.parms, clones=clones)
+        self.parms.update(parms)
 
     def resolveLinks(self, *pa, **kwa):
         """Resolve any links in associated parms for actors
            Should be overridden if use arguments
         """
         parms = self.actor.resolveLinks(**self.parms)
-        for key, value in parms.items():
-            self.parms[key] = value
+        self.parms.update(parms) 
 
 class Nact(Act):
     """lightweight container class for action references and parameters
@@ -141,10 +158,61 @@ class Actor(registering.StoriedRegistry):
     def expose(self):
         """Show Actor."""
         console.terse("Actor {0}".format(self.name))
+        
+    def clone(self):
+        """ Clone self in support of framer cloning.
+            Actors that are fully parametrized work as singletons so just return
+            self. Non fully parametrized actor classes need to override to make
+            copy of instance
+        """
+        return self
+       
+    def cloneParms(self, parms, clones, **kw):
+        """ Returns parms fixed up for framer cloning. This includes:
+            Reverting any Frame links to name strings,
+            Reverting non cloned Framer links into name strings
+            Replacing any cloned framer links with the cloned name strings from clones
+            Replacing any parms that are acts, in this case the needs with clones.
+            
+            clones is dict whose items keys are original framer names
+            and values are duples of (original,clone) framer references
+        """
+        parms = self.rerefShares(parms, clones)
+        return parms
+    
+    def rerefShares(self, parms, clones):
+        """ Returns parms fixed up for framer cloning.
+            clones is dict whose items keys are original framer names
+            and values are duples of (original,clone) framer references
+            
+            Generates oldPrefix and newPrefix from the original and new framer names
+            Replaces refs to shares whose path starts with oldPrefix
+                    with refs to new shares whose path starts with newPrefix.
+            The rest of the path stays the same.
+            Create new share if needed.
+            Deep copy the contents of the share.
+            This is to support framer cloning so typically the path prefixes
+            are framer relative paths.
+        """
+        for key, val in parms.items():
+            if isinstance(val, storing.Share):
+                for original, clone in clones.values():
+                    oldPrefix = "framer.{0}.".format(original.name)
+                    newPrefix = "framer.{0}.".format(clone.name)                
+                    if val.name.startswith(oldPrefix):
+                        newPath = val.name.replace(oldPrefix, newPrefix, 1)
+                        newShare = self.store.create(newPath)
+                        stuff = {}
+                        for k, v in val.items():
+                            stuff[k] = copy.deepcopy(v)
+                        newShare.create(stuff)
+                        parms[key] = newShare                    
+
+        return parms
     
     def resolveLinks(self, **kw):
-        """Resolves any links in parms for Actor
-           should be overridden by sub class
+        """ Resolves any links in parms for Actor
+            should be overridden by sub class
         """
         return {} #empty parms
 
@@ -236,15 +304,50 @@ class Transiter(Interrupter):
         framer.activate(active = far)
         return far
 
-
     def expose(self):
         """      """
         console.terse("Transiter {0}\n".format(self.name))
+        
+    def cloneParms(self, parms, clones, **kw):
+        """ Returns parms fixed up for framing cloning. This includes:
+            Reverting any Frame links to name strings,
+            Reverting non cloned Framer links into name strings
+            Replacing any cloned framer links with the cloned name strings from clones
+            Replacing any parms that are acts, in this case the needs with clones.
+            
+            clones is dict whose items keys are original framer names
+            and values are duples of (original,clone) framer references
+        """
+        parms = super(Transiter,self).cloneParms(parms, clones, **kw)
+        
+        needs = parms.get('needs')
+        near = parms.get('near')
+        far =  parms.get('far')
 
+        if needs:
+            needClones = []
+            for act in needs:
+                needClones.append(act.clone())
+            parms['needs'] = needClones
+        
+        if isinstance(near, framing.Frame):
+            parms['near'] = near.name # revert to name
+        
+        if isinstance(far, framing.Frame):
+            parms['far'] = far.name # revert to name        
+        
+        return parms    
+        
     def resolveLinks(self, needs, near, far, human, **kw):
         """Resolve any links in far and in associated parms for actors"""
 
         parms = {}
+        
+        if not isinstance(near, framing.Frame):
+            if near not in framing.Frame.Names:
+                raise excepting.ResolveError("ResolveError: Bad near link",
+                                            human, near)
+            parms['near'] = near = framing.Frame.Names[near] #replace name with valid link        
 
         if not isinstance(far, framing.Frame):
             if far == 'next':
@@ -263,7 +366,7 @@ class Transiter(Interrupter):
                 raise excepting.ResolveError("ResolveError: Bad far link",
                                              near.name + " " + human, far)
 
-        parms['far'] = far #replace name with valid link
+            parms['far'] = far #replace name with valid link
 
         for act in needs:
             act.resolveLinks()
@@ -357,22 +460,59 @@ class Suspender(Interrupter):
         aux.exitAll() # also sets .done = True
         aux.main = None
 
+    def cloneParms(self, parms, clones, **kw):
+        """ Returns parms fixed up for framing cloning. This includes:
+            Reverting any Frame links to name strings,
+            Reverting non cloned Framer links into name strings
+            Replacing any cloned framer links with the cloned name strings from clones
+            Replacing any parms that are acts, in this case the needs with clones.
+            
+            clones is dict whose items keys are original framer names
+            and values are duples of (original,clone) framer references
+        """
+        parms = super(Suspender,self).cloneParms(parms, clones, **kw)
+        
+        needs = parms.get('needs')
+        main = parms.get('main')
+        aux = parms.get('aux')
+        
+        if needs:
+            needClones = []
+            for act in needs:
+                needClones.append(act.clone())
+            parms['needs'] = needClones
+        
+        if isinstance(main, framing.Frame):
+            parms['main'] = main.name # revert to name
+               
+        if isinstance(aux, framing.Framer):
+            if aux.name in clones:
+                parms['aux'] = clones[aux.name][1].name #change to clone name
+            else:
+                parms['aux'] = aux.name # revert to name
+        elif aux: # assume namestring
+            if aux in clones:
+                parms['aux'] = clones[aux][1].name # change to clone name
+        
+        return parms
+
     def resolveLinks(self, needs, main, aux, human, **kw):
         """Resolve any links aux and in associated parms for actors"""
 
         parms = {}
+        
+        if not isinstance(main, framing.Frame):
+            if main not in framing.Frame.Names:
+                raise excepting.ResolveError("ResolveError: Bad main link", human, main)
+            parms['main'] = main = framing.Frame.Names[main] #replace name with valid link                    
 
         if not isinstance(aux, framing.Framer):
-            if aux in framing.Framer.Names:
-                aux = framing.Framer.Names[aux]
-
-            else:
-                raise excepting.ResolveError("ResolveError: Bad far link",
-                                             near.name + " " + human, far)
-
-        parms['aux'] = aux #replace name with valid link
-
-        console.terse("    Resolved aux as {0}".format(aux.name))
+            if aux not in framing.Framer.Names:
+                raise excepting.ResolveError("ResolveError: Bad aux link",
+                                                main.name + " " + human, aux)                
+                
+            parms['aux'] = aux = framing.Framer.Names[aux] #replace name with valid link
+            console.terse("    Resolved aux as {0} in {1}".format(aux.name, main.name))
 
         if aux.schedule != AUX:
             raise excepting.ResolveError("ResolveError: Bad aux link not scheduled as AUX",
@@ -390,7 +530,7 @@ class Deactivator(Actor):
        Deactivator is a special actor that acts on an Act
 
        In the Deactivator's action method, 
-          IF the act passed in as a parameter has a activate attribute THEN
+          IF the act passed in as a parameter has an _activate attribute THEN
                        it calls the deactivate method
        This is to ensure the outline in cond aux is exited cleanly when the aux
           is preempted by higher level transition.
@@ -416,30 +556,54 @@ class Deactivator(Actor):
         """
         #console.profuse("{0} action {1} for {2}\n".format(self.name, actor.name, aux.name))
 
-        if hasattr(actor, '_activative') and not  aux.done:
+        if hasattr(actor, '_activative') and not aux.done:
             console.profuse("{0} deactivate {1} for {2}\n".format(self.name, actor.name, aux.name))
             return actor.deactivate(aux)
 
     def expose(self):
         """   """
         console.terse("Deactivator {0}\n".format(self.name))
+        
+    def cloneParms(self, parms, clones, **kw):
+        """ Returns parms fixed up for framing cloning. This includes:
+            Reverting any Frame links to name strings,
+            Reverting non cloned Framer links into name strings
+            Replacing any cloned framer links with the cloned name strings from clones
+            Replacing any parms that are acts with clones.
+            
+            clones is dict whose items keys are original framer names
+            and values are duples of (original,clone) framer references
+        """
+        parms = super(Deactivator,self).cloneParms(parms, clones, **kw)
+        
+        actor = parms.get('actor')
+        aux = parms.get('aux')
+        
+        if isinstance(aux, framing.Framer):
+            parms['aux'] = aux.name # revert to name
+        
+        if isinstance(aux, framing.Framer):
+            if aux.name in clones:
+                parms['aux'] = clones[aux.name][1].name
+            else:
+                parms['aux'] = aux.name # revert to name
+        elif aux: # assume namestring
+            if aux in clones:
+                parms['aux'] = clones[aux][1].name        
 
+        return parms
+              
     def resolveLinks(self, actor, aux, **kw):
         """Resolve any links aux and in associated parms for actors"""
 
         parms = {}
 
         if not isinstance(aux, framing.Framer):
-            if aux in framing.Framer.Names:
-                aux = framing.Framer.Names[aux]
-
-            else:
+            if aux not in framing.Framer.Names:
                 raise excepting.ResolveError("ResolveError: Bad far link",
-                                             near.name + " " + human, far)
-
-        parms['aux'] = aux #replace name with valid link
-
-        console.terse("    Resolved aux as {0}".format(aux.name))
+                                                             near.name + " " + human, far)                
+            parms['aux'] = aux = framing.Framer.Names[aux] #replace name with valid link
+            console.terse("    Resolved aux as {0}".format(aux.name))
 
         if aux.schedule != AUX:
             msg = "ResolveError: Bad deactivator aux link not scheduled as AUX"
@@ -517,7 +681,35 @@ class Printer(Actor):
         """   """
         console.terse("Printer {0}\n".format(self.name))
         
-class UpdateMarker(Actor):
+class Marker(Actor):
+    """ Base class that sets up mark in provided share reference"""
+    
+    def cloneParms(self, parms, clones, **kw):
+        """ Returns parms fixed up for framing cloning. This includes:
+            Reverting any Frame links to name strings,
+            Reverting non cloned Framer links into name strings
+            Replacing any cloned framer links with the cloned name strings from clones
+            Replacing any parms that are acts with clones.
+            
+            clones is dict whose items keys are original framer names
+            and values are duples of (original,clone) framer references
+        """
+        parms = super(Marker,self).cloneParms(parms, clones, **kw)
+        
+        share = parms.get('share')
+        name = parms.get('name')
+        
+        #don't need to do anything yet since rerefShares will reref
+
+        return parms    
+    
+    def resolveLinks(self, share, name, **kw):
+        """Resolves share and adds mark in share"""
+        parms = {}
+            
+        return parms    
+        
+class UpdateMarker(Marker):
     """ UpdateMarker Class 
 
         UpdateMarker is a special actor that acts on a share to mark the update by
@@ -561,7 +753,7 @@ class UpdateMarker(Actor):
         """   """
         console.terse("UpdateMarker {0}\n".format(self.name))
         
-class ChangeMarker(Actor):
+class ChangeMarker(Marker):
     """ ChangeMarker Class 
 
         ChangeMarker is a special actor that acts on a share to mark save a copy
