@@ -48,7 +48,33 @@ class Complete(acting.Actor):
         if 'preface' not in kw:
             kw['preface'] = 'Complete'
 
-        super(Complete,self).__init__(**kw)  
+        super(Complete,self).__init__(**kw)
+        
+    def cloneParms(self, parms, clones, **kw):
+        """ Returns parms fixed up for framing cloning. This includes:
+            Reverting any Frame links to name strings,
+            Reverting non cloned Framer links into name strings
+            Replacing any cloned framer links with the cloned name strings from clones
+            Replacing any parms that are acts with clones.
+            
+            clones is dict whose items keys are original framer names
+            and values are duples of (original,clone) framer references
+        """
+        parms = super(Complete,self).cloneParms(parms, clones, **kw)
+        
+        framer = parms.get('framer')
+
+        if isinstance(framer, framing.Framer):
+            if framer.name in clones:
+                parms['framer'] = clones[framer.name][1].name
+            else:
+                parms['framer'] = framer.name # revert to name
+        elif framer: # assume namestring
+            if framer in clones:
+                parms['framer'] = clones[framer][1].name   
+        
+        return parms
+           
 
     def resolveLinks(self, framer, **kw):
         """Resolves value (framer) link that is passed in as parm
@@ -56,13 +82,10 @@ class Complete(acting.Actor):
            since framer may not be current framer at build time
         """
         parms = {}
-        if not isinstance(framer, framing.Framer): #so name of framer
-            if framer not in framing.Framer.Names: 
-                raise excepting.ResolveError("ResolveError: Bad done framer link name", framer, '')
-            framer = framing.Framer.Names[framer]
-
-        if not isinstance(framer, framing.Framer): #maker sure framer not other tasker
-            raise excepting.ResolveError("ResolveError: Bad done framer name not for framer", framer, '')
+        parms['framer'] = framer = framing.resolveFramer(framer, who=self.name)
+        
+        #if not isinstance(framer, framing.Framer): #maker sure framer not other tasker
+            #raise excepting.ResolveError("ResolveError: Bad done framer name not for framer", framer, '')
 
         #if not framer.schedule in [AUX, SLAVE]: #maker sure framer is auxliary or slave
             #raise excepting.ResolveError("ResolveError: Bad done framer, framer not auxiliary or slave", framer, '')
@@ -88,7 +111,7 @@ class DoneComplete(Complete):
 
         """
         framer.done = True
-        console.profuse("    Done {0}\n".format(self.name))
+        console.profuse("    Done {0}\n".format(framer.name))
 
         return None
 
