@@ -8,6 +8,7 @@ import time
 import struct
 from collections import deque,  Mapping
 import inspect
+import copy
 
 from .globaling import *
 from .odicting import odict
@@ -70,7 +71,6 @@ class Deed(acting.Actor):
         self.ioinit = ioinit or odict() #dict with default ioinit arguments
         self._iois = odict() # attribute names inited with .initio if any
 
-        
     def preinitio(self, **kw):
         """ Parse time Reinit
             Enables initializing instance at parse time from FloScript options
@@ -85,6 +85,9 @@ class Deed(acting.Actor):
             The other argument values are the pathname strings of Deed
             specific io shares or nodes
             
+            This provides support for the 'per' and 'for' connectives in the do
+            command
+            
             The inital values of the shares must be performed somewhere else
             either in the .postinitio method of the Deed or by a FloScript command
             such as init or put
@@ -92,9 +95,6 @@ class Deed(acting.Actor):
             This allows dynamic initialization of Deed instances at FloScript
             parse time not python module load time.
             
-            Need to change signatue so can pass in odict as pa or list of tuples
-            like the update method for shares
-            
             Because preinitio is executed at parse time by the builder when deed
             appears in FloScript, preinitio will override the default values
             set in .ioint (if any) when its updated in CreateInstances.
@@ -104,35 +104,115 @@ class Deed(acting.Actor):
         ioinit.update(kw)
         return ioinit
     
-    def initio(self, inode="", **kw):
+    def relinitio(self, framer, frame, inode=""):
+        """ Return realtive inode ipath if inode's irel is given
+            with frame or framer relative path string
+            This is called by the builder to enable frame or framer relative inodes
+            
+            There are the following 3 formats for the value of the inode argument:
+            
+            1- string
+               ipath = pathnamestring
+            
+            2- dict of key: values (mapping)
+            
+                {
+                    ipath: "pathnamestring",
+                    irel: relativity, "frame" or "framer" 
+                }
+            
+            2- tuple or list of values (non mapping non string iterable)
+            
+               (ipath, irel) or [ipath, irel]
+               ipath = pathnamestring
+               irel = relativity,  "frame" or "framer" 
+            
+            
+            In all cases, two init values are produced, these are:
+                ipath, irel
+            
+            When specifying inode in the associated ioinit dict the value of irel is
+            either "frame" or "framer". This method will subsitute correspondingly
+            the full relative prefix for the associated Frame or Framer and return it.
+            
+            For inode, the ipath value is the pathname string of the share node
+                where shares associated with the Deed instance may be placed.
+                
+            If the inode ipath is empty then the default value for ipath
+              will be created from the instance name where uppercase letters
+              indicate intermediate nodes.
+              
+            When irel in ioinit is "frame" then the full inode ipath will be frame relative
+                (The frame in the framer where the Deed appears) and the passed
+                in value of irel will be  "framer.framername.frame.framername"
+            
+            When irel in ioinit is "framer" the the full inode ipath will be framer relative
+                 (the framer where the Deed appears)  and the passed in value of irel
+                 will be "framer.framername"
+                     
+                
+            Examples:
+                If the inode argument is ("test.detail", 'frame')
+                Then the inode ipath would be "framer.framername.frame.framename.test.detail"
+                     
+                If the inode argument is "" and the Deed instance is named 'thingGoneWrong' 
+                Then the inode ipath would be "thing.gone.wrong".
+            
+                If the inode argument value is ("", "framer")
+                Then the inode ipath would be  "framer.framername.thing.gone.wrong"
+            
+        """
+        if isinstance(inode, basestring):
+            ipath = inode
+            irel = None
+            
+        elif isinstance(inode, Mapping): # dictionary
+            ipath = inode.get('ipath')
+            irel = inode.get('irel')                
+        
+        elif isinstance(inode, NonStringIterable): # non dict and non string iterable
+            ipath, irel = just(2, inode)
+                
+        else:
+            raise ValueError("Bad inode arg '{0}'".format(inode))
+        
+        if not ipath:
+            inode = nameToPath(self.name)
+        
+        if irel:
+            if irel == "frame":
+                ipath = '.'.join(['framer', framer.name, 'frame', frame.name, ipath.lstrip('.')])
+            elif irel == "framer":
+                ipath = '.'.join(['framer', framer.name, ipath.lstrip('.')])
+            else:
+                raise ValueError("Bad irel in inode arg '{0}'".format(inode))
+        
+        return ipath
+            
+    def initio(self, inode, **kw):
         """ Intialize and hookup ioflo shares from node pathname inode and kw arguments.
             This implements a generic Deed interface protocol for associating the
             io data flow shares to the Deed.
             
-            The 'inode' argument is a pathname string of the share node for the instance
-            where associated shares may be placed. If inode is empty then the default
-            value for inode will be created from the instance name where uppercase
-            letters indicate intermediate nodes.
+            inode is the computed pathname string of the share node where shares associated
+            with the Deed instance may be placed for use by the other kw arguments.
+            inode is computed by relinitio to allow for frame and framer relative inodes
             
-            For example an Deed instance named 'thingGoneWrong' would have a default
-            inode of "thing.gone.wrong".
+            For the syntax of specifying inode in the ioinit dict see the docstring
+            for relinitio
             
-            The values of other items in the **kw argument may be either strings,
+            The values of the items in the **kw argument may be either strings,
             non-string iterables such as list or tuple, or named iterables such as
             namedtuple or dicts.
             
             The init behavior of the other arguments is based on the form of the argument value.
             
-            There are the following 3 forms:
+            There are the following 3 forms for val:
             
             1- string
-               ipath  pathnamestring
+               ipath = pathnamestring
             
-            2- tuple of values
-            
-               (ipath, ival, iown)
-            
-            2- dict of key: values
+            2- dict of key: values (mapping)
             
                 {
                     ipath: "pathnamestring",
@@ -140,6 +220,13 @@ class Deed(acting.Actor):
                     iown: truthy,
                 }
             
+            2- tuple or list of values (non mapping non string iterable)
+            
+               (ipath, ival, iown) or [ipath, ival, iown]
+               ipath = pathnamestring
+               ival = initial value
+               iown = truthy
+            
             
             In all cases, three init values are produced, these are:
                 ipath, ival, iown,
@@ -149,23 +236,39 @@ class Deed(acting.Actor):
             The following rules are applied when given the values of
                ipath, ival, iown
             
-            If argument 'inode' provided:
-               inode is assigned the item value which is a pathname string
-            Else
-               inode is derived from the Deed instance name
-            
-            Assign .inode attribute with value of inode node
             
             For each kw item (key, val)
                 key is the name of the associated instance attribute.
+                Extract ipath, ival, iown from val
                 
-                If ival not provided do not initialize share leave as is
+                Shares are initialized with mappings passed into share.create or
+                share.update. So to assign ival to share.value pass into share.create
+                or share.update a mapping of the form {'value': ival} whereas
+                passing in an empty mapping does nothing.
                 
-                If ival provided and is not a non-empty Mapping Then
-                   assign ival as value of share
-                   # This means there is no way to init a share.value to a non empty mapping
+                If ival not provided
+                     set ival to empty mapping which when passed to share.create
+                     will not change share.value
+                
+                Else ival provided
+                
+                    If ival is an empty Mapping Then
+                        assign a shallow copy of ival to share.value by passing
+                        in {value: ival (copy)} to share.create 
                    
-                Otherwise each item in ival is assigned as a field, value in the share 
+                    Else If ival is a non-string iterable and not a mapping
+                        assign a shallow copy of ival to share.value by passing
+                        in {value: ival (copy)} to share.create 
+                   
+                    Else If ival is a non-empty Mapping Then
+                        Each item in ival is assigned as a field, value pair in the share
+                        by passing ival directly into share.create
+                        This means there is no way to init a share.value to a non empty mapping
+                        It is possible to init a share.value to an empty mapping see below
+                    
+                    Else
+                        assign ival to share.value by by passing
+                        in {value: ival} to share.create 
 
                 Create share with pathname given by ipath
                     If ipath is provided
@@ -191,8 +294,8 @@ class Deed(acting.Actor):
                 
 
         """
-        if not inode:
-            inode = nameToPath(self.name)
+        if not isinstance(inode, basestring):
+            raise ValueError("Nonstring inode arg '{0}'".format(inode))           
         
         _parametric = hasattr(self, "_parametric")
         iois = odict()
@@ -223,10 +326,17 @@ class Deed(acting.Actor):
                     ival = odict() # effectively will not change share
                 else:
                     ival = val['ival']
-                
-                if not (ival and isinstance(ival, Mapping)): #not non-empty mapping
-                    ival = odict(value=ival)
-                #else: #ival = ival
+                    
+                    if isinstance(ival, Mapping):
+                        if not ival: #empty mapping
+                            ival = odict(value=copy.copy(ival)) #make copy so each instance unique
+                        # otherwise don't change since ival is non-empty mapping
+                        
+                    elif isinstance(ival, NonStringIterable): # not mapping and NonStringIterable
+                        ival = odict(value=copy.copy(ival))
+                    
+                    else: 
+                        ival = odict(value=ival)                    
             
             elif isinstance(val, NonStringIterable): # non dict non string iterable
                 length = len(val)
@@ -244,31 +354,35 @@ class Deed(acting.Actor):
                     ival = odict() # effectively will not change share
                 else:
                     ival = val[1]
-                    if not (ival and isinstance(ival, Mapping)): #not non-empty mapping
+                    if isinstance(ival, Mapping):
+                        if not ival: #empty mapping
+                            ival = odict(value=copy.copy(ival)) #make copy so each instance unique
+                        # otherwise don't change since ival is non-empty mapping
+                        
+                    elif isinstance(ival, NonStringIterable): # not mapping and NonStringIterable
+                        ival = odict(value=copy.copy(ival))
+                    
+                    else: 
                         ival = odict(value=ival)
-                    #else:  #ival = ival
             
             else:
                 raise ValueError("Bad init kw arg '{0}'with Value '{1}'".format(key, val))
-                
+            
             
             if ipath:
                 if not ipath.startswith('.'): # full path is inode joined to ipath
                     ipath = '.'.join((inode.rstrip('.'), ipath)) # when inode empty prepends dot
-                
-                if ipath.endswith('.'): #init a node not share not in iflos or oflos
-                    ipath = ipath.rstrip('.') # remove trailing dot
-                    setattr(self, key, self.store.createNode(ipath))
-                    continue
-                    
             else:
-                ipath = '.'.join(inode, key)
+                ipath = '.'.join(inode.rstrip('.'), key)
             
-            ioi = self.store.create(ipath)
-            if not iown:
-                ioi.create(ival)
+            if ipath.endswith('.'): #init a node not share ignore ival
+                ioi = self.store.createNode(ipath.rstrip('.'))
             else:
-                ioi.update(ival)           
+                ioi = self.store.create(ipath)
+                if not iown:
+                    ioi.create(ival)
+                else:
+                    ioi.update(ival)           
             
             if _parametric:
                 iois[key] = ioi
@@ -287,7 +401,6 @@ class Deed(acting.Actor):
         
         return iois # if non-empty then treat as parametric
     
-    
     def postinitio(self):
         """ Base method to be overriden in sub classes. Perform post initio setup"""
         pass
@@ -417,12 +530,10 @@ class LapseDeed(Deed):
         console.profuse("Actioning LapseDeed  {0}\n".format(self.name))
         self.updateLapse()
 
-
     def expose(self):
         """     """
         print "Deed %s stamp = %s lapse = %s" % (self.name, self.stamp, self.lapse)
-
-
+        
 def Test():
     """Module Common self test
 
