@@ -7,7 +7,7 @@ import copy
 from collections import deque
 from itertools import izip
 
-
+from .odicting import odict
 from .globaling import *
 from . import excepting
 from . import registering
@@ -17,47 +17,77 @@ from . import tasking
 from .consoling import getConsole
 console = getConsole()
 
+#utility functions
+def resolveFramer(framer, who=''):
+    """ Returns resolved framer instance from framer
+        framer may be name of framer or instance
+        who is optinal name of object owning the link such as framer or frame
+        Framer.Names registry must be setup
+    """
+    if not isinstance(framer, Framer): # not instance so name
+        if framer not in Framer.Names: 
+            raise excepting.ResolveError("ResolveError: Bad framer link name", framer, who)
+        framer = Framer.Names[framer]
+
+    return framer
+
+ResolveFramer = resolveFramer
+
+def resolveFrame(frame, who=''):
+    """ Returns resolved frame instance from frame
+            frame may be name of frame or instance
+    
+            Frame.Names registry must be setup
+        """    
+    if not isinstance(frame, Frame): # not instance so name
+        if frame not in Frame.Names: 
+            raise excepting.ResolveError("ResolveError: Bad frame link name", frame, who)
+        frame = Frame.Names[frame] #replace frame name with frame
+    
+    return frame
+
+ResolveFrame = resolveFrame
 
 #Class definitions
 
 class Framer(tasking.Tasker):
-    """Framer Task Patron Registry Class for running hierarchical action framework
+    """ Framer Task Patron Registry Class for running hierarchical action framework
 
-       inherited instance attributes
-          .name = unique name for machine
-          .store = data store
-
-          .period = desired time in seconds between runs must be non negative, zero means asap
-          .stamp = time of last outline change beginning time to compute elapsed time
-          .status = operational status of tasker
-          .desire = desired control asked by this or other taskers
-          .done = tasker completion state True or False
-          .schedule = scheduling context of this Task for Skedder
-          .runner = generator to run tasker
+        inherited instance attributes
+            .name = unique name for machine
+            .store = data store
+  
+            .period = desired time in seconds between runs must be non negative, zero means asap
+            .stamp = time of last outline change beginning time to compute elapsed time
+            .status = operational status of tasker
+            .desire = desired control asked by this or other taskers
+            .done = tasker completion state True or False
+            .schedule = scheduling context of this Task for Skedder
+            .runner = generator to run tasker
 
        instance attributes
-          .main = main frame when this framer is an auxiliary
-          .done = auxiliary completion state True or False when an auxiliary
-          .elapsed = elapsed time from outline change
-          .elapsedShr = share where .elapsed is stored for logging and need checks
-
-          .recurred = number of completed recurrences of the current outline
-                   recurred is zeroed upon entry
-                   during first iteration recurred is 0 
-                   during second iteration (before trans evaluated) recurred is 1
-                   so a trans check on recurred == 2 means its already iterated twice
-          .recurredShr = share where recurred is stored for logging and need checks
-
-          .first = first frame (default frame to start at)
-          .active = active frame
-          .actives = active outline list of frames
-          .activeShr = share where .active name is stored for logging
-          
-          .human = human readable version of active outline
-          .humanShr = share where .human is stored for logging
-
-          .frameNames = frame name registry , name space of frame names
-          .frameCounter = frame name registry counter 
+            .main = main frame when this framer is an auxiliary
+            .done = auxiliary completion state True or False when an auxiliary
+            .elapsed = elapsed time from outline change
+            .elapsedShr = share where .elapsed is stored for logging and need checks
+  
+            .recurred = number of completed recurrences of the current outline
+                     recurred is zeroed upon entry
+                     during first iteration recurred is 0 
+                     during second iteration (before trans evaluated) recurred is 1
+                     so a trans check on recurred == 2 means its already iterated twice
+            .recurredShr = share where recurred is stored for logging and need checks
+  
+            .first = first frame (default frame to start at)
+            .active = active frame
+            .actives = active outline list of frames
+            .activeShr = share where .active name is stored for logging
+  
+            .human = human readable version of active outline
+            .humanShr = share where .human is stored for logging
+  
+            .frameNames = frame name registry , name space of frame names
+            .frameCounter = frame name registry counter 
     """
     #Counter = 0  
     #Names = {}
@@ -92,15 +122,51 @@ class Framer(tasking.Tasker):
         path = 'framer.' + self.name + '.state.active'
         self.activeShr = self.store.create(path)
         self.activeShr.update(value = self.active.name if self.active else "")
-        
+
         self.human = '' #human readable version of actives outline
         path = 'framer.' + self.name + '.state.human'
         self.humanShr = self.store.create(path)
         self.humanShr.update(value = self.human)        
-        
+
         self.frameNames = {} #frame name registry for framer. name space of frame names
         self.frameCounter = 0 #frame name registry counter for framer
+    
+    def clone(self, index, clones):
+        """ Return clone of self with name derived from index
+            Assumes that Framer Registry as been assigned to self.store.house
+            
+        """
+        name = "{0}_{1:d}".format(self.name, index)
+        
+        if name and not REO_IdentPub.match(name):
+            msg = "CloneError: Invalid framer name '{0}'.".format(name)
+            raise excepting.CloneError(msg)  
+        
+        if name in Framer.Names:
+            msg = "CloneError: Framer '{0}' already exists.".format(name)
+            raise excepting.CloneError(msg)
+                
+        clone = Framer(name=name, store=self.store, period=0.0)
+        console.profuse("     Cloning framer {0} to {1}\n".format(self.name, clone.name))
+        clone.schedule = AUX
+        clone.first = self.first.name # resolve later
+        clones[self.name] = (self, clone)
+        
+        for frame in self.frameNames.values():
+            for aux in frame.auxes:
+                aux.clone(index, clones) # changes clones in place
 
+        return clone
+
+    def cloneFrames(self, clone, clones):
+        """ Return clone of self with name derived from index
+            Assumes that Framer Registry as been assigned to self.store.house
+            
+        """
+        clone.assignFrameRegistry()
+        for frame in self.frameNames.values():
+            frame.clone(framer=clone, clones=clones) #creates cloned frames in cloned framer registry
+        
     def assignFrameRegistry(self):
         """Point Frame class name registry dict and counter to .frameNames
            and .frameCounter.
@@ -110,7 +176,6 @@ class Framer(tasking.Tasker):
         Frame.Names = self.frameNames
         Frame.Counter = self.frameCounter
 
-
     def restartTimer(self):
         """reset the start time and elapsed time of framer for changed outline
 
@@ -163,7 +228,6 @@ class Framer(tasking.Tasker):
             self.recurredShr.name, self.recurredShr.value, self.recurred))
         self.recurredShr.update(value = self.recurred)
 
-
     def resolveLinks(self):
         """Convert all the name strings for links to references to instance
            by that name
@@ -187,7 +251,7 @@ class Framer(tasking.Tasker):
 
 
     def traceOutlines(self):
-        """Trace and assign outlines for each frame in framework
+        """Trace and assign outlines for each frame in framer
         """
         print "Tracing outlines for framer %s" % (self.name)
 
@@ -240,11 +304,10 @@ class Framer(tasking.Tasker):
         return self.checkEnter(self.first.outline)
 
     def checkEnter(self, enters = []):
-        """checks entryNeeds for frames in enters list
+        """checks beacts for frames in enters list
            return on first failure do not keep testing
            assumes enters outline in top down order
-
-
+           
         """
         console.profuse("{0}Check enters of {1} Framer {2}\n".format(
             '    ' if self.schedule == AUX or self.schedule == SLAVE else '',
@@ -618,34 +681,32 @@ class Framer(tasking.Tasker):
 
 
 class Frame(registering.StoriedRegistry):
-    """Frame Class for hierarchical action framework object
+    """ Frame Class for hierarchical action framework object
 
-       inherited instance attributes
-          .name = unique name for frame
-          .store = data store
+        inherited instance attributes
+            .name = unique name for frame
+            .store = data store
 
-       instance attributes
-          .framer = link to framer that executes this frame
-          .over = link to frame immediately above this one in hierarchy
-          .under = property link to primary frame immediately below this one in hierarchy 
-          .unders = list of all frames immediately below this one
-          .outline = list of frames in outline for this frame top down order 
-          .head = list of frames from top down to self
-          .human = string of names of frames in outline top down '>' separated
-          .headHuman = string of names of frames in head top down '>' separated
-          .next = next frame used by builder for transitions to next
-
-          .beacts = before entry action (need) acts or entry checks
-          .preacts = precur action acts (pre transition recurrent actions and transitions)
-          #.transacts = trans action acts
-          .enacts = enter action acts
-          .renacts = renter action acts
-          .reacts = recur action acts
-          .exacts = exit action acts
-          .rexacts = rexit action acts
-
-          .auxes = auxiliary framers
-          .specs = specs for auxilary framers
+        instance attributes
+            .framer = link to framer that executes this frame
+            .over = link to frame immediately above this one in hierarchy
+            .under = property link to primary frame immediately below this one in hierarchy 
+            .unders = list of all frames immediately below this one
+            .outline = list of frames in outline for this frame top down order 
+            .head = list of frames from top down to self
+            .human = string of names of frames in outline top down '>' separated
+            .headHuman = string of names of frames in head top down '>' separated
+            .next = next frame used by builder for transitions to next
+  
+            .beacts = before entry action (need) acts or entry checks
+            .preacts = precur action acts (pre transition recurrent actions and transitions)
+            .enacts = enter action acts
+            .renacts = renter action acts
+            .reacts = recur action acts
+            .exacts = exit action acts
+            .rexacts = rexit action acts
+  
+            .auxes = auxiliary framers
 
     """
     Counter = 0  
@@ -678,7 +739,65 @@ class Frame(registering.StoriedRegistry):
         self.rexacts = [] #list of re-exit acts callables upon re-exit
 
         self.auxes = [] #list of auxilary framers for this frame
-        self.specs = {} #dictionary of parameters for auxilary framers
+
+    def clone(self, framer, clones):
+        """ Return clone of self by creating new frame in framer and by 
+            frame links, acts, and auxes
+            clones is dict with items each key is name of orignal framer and value
+                 is duple of (original, clone) framer references
+            
+            Assumes that the Frame Registry is pointing to framer which is a clone
+            of this Frame's Framer so all new Frames will be in the cloned registry.
+
+        """
+        clone = Frame(  name=self.name,
+                        store=self.store, 
+                        framer=framer)
+        console.profuse("     Cloning frame {0} into framer {1}\n".format(
+                               clone.name, framer.name))
+        
+        if self.over:
+            if isinstance(self.over, Frame):
+                clone.over = self.over.name            
+            else:
+                clone.over = self.over 
+        if self.next:
+            if isinstance(self.next, Frame):
+                clone.next = self.next.name            
+            else:
+                clone.next = self.next
+        
+        for under in self.unders:
+            if isinstance(under, Frame):
+                clone.unders.append(under.name)
+            else:
+                clone.unders.append(under)
+                
+        for i, aux in enumerate(self.auxes): #replace each aux with its clone name
+            if isinstance(aux, Framer):
+                if aux.name in clones:
+                    self.auxes[i] = clones[aux.name][1].name                
+            else: # assume namestring
+                if aux in clones:
+                    self.auxes[i] = clones[aux][1].name
+            
+        for act in self.beacts:
+            clone.addBeact(act.clone(clones)) 
+        for act in self.preacts:
+            clone.addPreact(act.clone(clones))
+        for act in self.enacts:
+            clone.addEnact(act.clone(clones))
+        for act in self.renacts:
+            clone.addRenact(act.clone(clones))
+        for act in self.reacts:
+            clone.addReact(act.clone(clones))
+        for act in self.exacts:
+            clone.addExact(act.clone(clones))
+        for act in self.rexacts:
+            clone.addRexact(act.clone(clones))
+
+        
+        return clone
 
     def expose(self):
         """Prints out instance variables.
@@ -692,7 +811,6 @@ class Frame(registering.StoriedRegistry):
         print "name = %s, framer = %s, over = %s, under = %s" % \
               (self.name, framername, self.over, self.under)
 
-
     def getUnder(self):
         """getter for under property
 
@@ -716,7 +834,6 @@ class Frame(registering.StoriedRegistry):
 
     under = property(fget = getUnder, fset = setUnder, doc = "Primary under frame")
 
-
     def detach(self):
         """detach self from .over. Fix under links in .over
 
@@ -726,7 +843,6 @@ class Frame(registering.StoriedRegistry):
                 self.over.unders.remove(self)
             self.over = None
 
-
     def attach(self, over):
         """attaches self to over frame if attaching would not create loop
 
@@ -757,7 +873,6 @@ class Frame(registering.StoriedRegistry):
                 frame = frame.over #keep going up outline
         return False #no loop
 
-
     def resolveNextLink(self):
         """Resolve next link
 
@@ -772,7 +887,7 @@ class Frame(registering.StoriedRegistry):
                 self.next = next
 
     def resolveOverLinks(self):
-        """Starting with self climb over links resolving the links as needed along the way
+        """Starting with self.over climb over links resolving the links as needed along the way
 
         """
         over = self.over
@@ -806,11 +921,9 @@ class Frame(registering.StoriedRegistry):
 
             under = over
             over = over.over #rise one level
-
+            
     def resolveUnderLinks(self):
-        """Resolve under links
-
-        """
+        """ Resolve under links """
         for i, under in enumerate(self.unders):
             if not isinstance(under, Frame):
                 try:
@@ -822,11 +935,8 @@ class Frame(registering.StoriedRegistry):
         #with right most removed
         #or at least check for duplicates and flag error
 
-
     def resolveAuxLinks(self):
-        """Resolve aux links
-
-        """
+        """ Resolve aux links """
         for i, aux in enumerate(self.auxes):
             if not isinstance(aux, Framer): # link is name not object
                 if aux not in Framer.Names:
@@ -842,10 +952,7 @@ class Frame(registering.StoriedRegistry):
             #aux.main  is set upon frame.enter before aux.enterAll() so can reuse aux in other frames
 
     def resolveFramerLink(self):
-        """Resolve framer link
-
-        """
-
+        """Resolve framer link """
         if self.framer:
             if not isinstance(self.framer, Framer):
                 if self.framer not in Framer.Names:
@@ -854,15 +961,14 @@ class Frame(registering.StoriedRegistry):
                 if not isinstance(framer, Framer):  #maker sure framer not tasker since tasker framer share registry
                     raise excepting.ResolveError("ResolveError: Bad framer name, tasker not framer", self.name, framer.name)
                 self.framer = framer #replace link name with link
-
-
-
+        
     def resolveLinks(self):
         """Resolve links where links are instance name strings assigned during building
            need to be converted to object references using instance name registry
 
         """
-        console.profuse("Resolving links for frame {0}\n".format(self.name))
+        console.profuse("Resolving links for frame {0} in framer {1}\n".format(
+            self.name, self.framer.name))
 
         self.resolveFramerLink()
 
@@ -881,7 +987,6 @@ class Frame(registering.StoriedRegistry):
 
         for act in self.preacts:
             act.resolveLinks()
-            #act.resolveLinks(frame = self) #pass in frame
 
         for act in self.exacts:
             act.resolveLinks()
@@ -999,7 +1104,7 @@ class Frame(registering.StoriedRegistry):
         return human 
 
     def checkEnter(self):
-        """Check Entry Needs for self  and auxes
+        """Check beacts for self and auxes
         """
         console.profuse("    Check enter into {0}\n".format(self.name))
 
@@ -1014,7 +1119,7 @@ class Frame(registering.StoriedRegistry):
                 console.profuse("    False aux {0}.main in use".format(aux.name))
                 return False
 
-            if not aux.checkStart(): #performs entry checks
+            if not aux.checkStart(): #performs entry checks beacts
                 return False
 
         console.profuse("    True all {0}\n".format(self.name))
@@ -1118,7 +1223,7 @@ class Frame(registering.StoriedRegistry):
     def addEnact(self, act):
         """         """
         self.enacts.append(act)
-    
+
     def insertEnact(self, act, index=0):
         """         """
         self.enacts.insert(index, act)    
@@ -1292,4 +1397,3 @@ def Test():
 
 if __name__ == "__main__":
     Test()
-
