@@ -3,10 +3,10 @@
 """
 #print "\nPackage at%s" % __path__[0]
 
-__all__ = ['salting']
+__all__ = ['cloning', 'salting']
 
 for m in __all__:
     exec "from . import %s" % m  #relative import
 
 #used by CreateAllInstances                 
-_InstanceModules = [salting]
+_InstanceModules = [cloning, salting]
